import {
  Http,
  IEventParam,
  Metadata,
  Singleton,
  IDictionary,
  I18n,
} from '@zeedhi/core';
import {
  Menu, MenuButton, AlertService, Select, DialogService,
} from '@zeedhi/common';
import { TekLibAuth } from '@zeedhi/tek-lib';
import { TekUserInfo } from '@zeedhi/teknisa-components-common';
import { useMirage } from '../server/use-mirage';
import store from '../store';

@Singleton
export class AppController {
  public hasImage: boolean = false;

  get notHasImage(): boolean {
    return !this.hasImage;
  }

  public imagePath: string = '';

  public microservicePage: boolean = false;

  public toggleMenu({ component }: IEventParam<MenuButton>) {
    const menu = Metadata.getInstance<Menu>(component.menuName);
    menu.toggleMenu();
    if (menu.opened && !menu.miniState) {
      if (menu.showSearch) {
        setTimeout(() => {
          const searchInput = document.querySelector('.zd-menu-search-input input');
          if (searchInput) (searchInput as HTMLElement).focus();
        }, 100);
      } else {
        const firstMenuItem = document.querySelector('.zd-menu-link, .zd-menu-g > div');
        if (firstMenuItem) (firstMenuItem as HTMLElement).focus();
      }
    }
  }

  public metaLanguages: IDictionary = {
    pt_br: 'pt-BR',
    es_cl: 'es-CL',
    es_es: 'es-ES',
    es_co: 'es-CO',
    en_us: 'en-US',
  };

  public get nmFilialLogada() {
    const filial = Metadata.getInstance<Select>('CDFILIALLOGADA');
    return filial.value ? (filial.value as IDictionary).NMFILIAL : '';
  }

  public get cdFilialLogada() {
    return store.getters.cdFilialLogada;
  }

  get userName() {
    const username = localStorage.getItem('LOGIN_USER_NAME') || '';
    return this.getFormattedName(username);
  }

  get nameInitialLetter() {
    const name = this.userName;
    return name.slice(0, 1);
  }

  get logo() {
    return '/assets/img/logo.png';
  }

  public showMessageExpiredMenu() {
    TekLibAuth.showMessageExpiredMenu();
  }

  public closeMenu({ component }: IEventParam<MenuButton>) {
    const menu = Metadata.getInstance<Menu>(component.menuName);
    if (menu.opened && !menu.miniState) {
      menu.toggleMenu();
    }
  }

  public escAction() {
    AlertService.hide();
  }

  public async userInfoOnCreated() {
    const userInfoComponent = Metadata.getInstance<TekUserInfo>('userInfo');
    const currentModule = this.getCurrentModule();

    const { data } = await Http.get('/lib_generalInfo', { params: { currentModule } });
    userInfoComponent.versionInfo = data.versionInfo;

    if (data.userInfo.IMAGE) {
      this.hasImage = true;
      this.imagePath = data.userInfo.IMAGE;
    } else {
      this.hasImage = false;
    }
  }

  public userInfoOnLogoutClick() {
    // TekLibAuth.logoutConfirm();
    DialogService.show({
      name: 'logoutConfirm',
      type: 'warning',
      title: I18n.translate('scm::ATTENTION'),
      text: I18n.translate('scm::ALE_LOGOUT'),
      buttons: [
        {
          name: 'no',
          component: 'ZdButton',
          label: I18n.translate('NO'),
          outline: true,
          events: {
            click: () => {
              DialogService.hide();
            },
          },
        },
        {
          name: 'yes',
          component: 'ZdButton',
          label: I18n.translate('YES'),
          events: {
            click: () => {
              DialogService.hide();
              useMirage(false);
              TekLibAuth.logout();
            },
          },
        },
      ],
    });
    // TekLibMessageServices.showMessage('ALE_LOGOUT', {}, 'warning', buttonsDoLogout);
  }

  private getFormattedName(username: string): string {
    const usernameLowerCase = username.toLowerCase();
    const name = usernameLowerCase.replace(/(^\w{1})|(\s+\w{1})/g, (letter: string) => letter.toUpperCase());
    const splitName = name.split(' ');
    if (splitName.length > 1) {
      const firstName = splitName.shift();
      const lastName = splitName.pop();
      let middleName = '';

      splitName.forEach((item) => {
        middleName += ` ${item.slice(0, 1)}. `;
      });

      return `${firstName} ${middleName} ${lastName}`;
    }

    return name;
  }

  private getCurrentModule(): string {
    const { href } = window.location;
    let currentModule = 'product';
    if (href.includes('/modules/')) {
      let auxHref = href.split('modules/');
      auxHref = auxHref[1].split('/');
      [currentModule] = auxHref;
    }

    return currentModule;
  }

  public setInfoUnidadeLogada({ component }: IEventParam<Select>) {
    const cdfilial = component.value ? component.value.CDFILIAL : null;
    const cdfillanca = component.value ? component.value.CDFILLANCA : null;
    const cdfilcentcmp = component.value ? component.value.CDFILCENTCMP : null;
    localStorage.setItem('CDFILIALLOGADA', cdfilial);
    localStorage.setItem('CDFILLANCA', cdfillanca);
    localStorage.setItem('CDFILCENTCMP', cdfilcentcmp);
    store.commit('cdFilialLogada', cdfilial);
    component.validate();
  }

  public setUnidadeLogadaMounted = () => {
    if (localStorage.getItem('LOGIN_TOKEN')) {
      const cdfilial = store.getters.cdFilialLogada;
      const component = Metadata.getInstance<Select>('CDFILIALLOGADA');
      if (localStorage.getItem('hasImportedData') === 'true') {
        useMirage(true);
        component.datasource.filter = {
          CDOPERADOR: localStorage.getItem('CDOPERADOR') || '',
        };
      } else {
        useMirage(false);
        component.datasource.filter = {
          CDOPERADOR: localStorage.getItem('CDOPERADOR') || '',
          CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || 'T',
        };
      }
      component.value = cdfilial;
      // this.unidadeLogadaIsValid();
    }
  };

  public unidadeLogadaIsValid() {
    if (localStorage.getItem('LOGIN_TOKEN')) {
      const filialLogada = store.getters.cdFilialLogada;
      const filial = Metadata.getInstance<Select>('CDFILIALLOGADA');
      if (filialLogada === null || filialLogada === 'null' || !filial.value) {
        DialogService.hide();
        DialogService.show(
          {
            name: 'paramUnidadeLogada',
            type: 'warning',
            title: I18n.translate('scm::ATTENTION'),
            text: I18n.translate('scm::SELECT_UNITY'),
            buttons: [
              {
                name: 'dialogButton',
                label: I18n.translate('scm::OK'),
                component: 'ZdButton',
                events: { click: () => DialogService.hide() },
              },
            ],
          },
        );
      }
    }
  }

  public reloadDatasetFiliais({ component }: IEventParam<Select>) {
    if (localStorage.getItem('hasImportedData') === 'true') {
      component.datasource.filter = {
        CDOPERADOR: localStorage.getItem('CDOPERADOR') || '',
      };
    } else {
      component.datasource.filter = {
        CDOPERADOR: localStorage.getItem('CDOPERADOR') || '',
        CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || 'T',
      };
    }
    component.datasource.get();
  }

  public get getFilterFiliais() {
    if (localStorage.getItem('hasImportedData') === 'true') {
      return {
        CDOPERADOR: localStorage.getItem('CDOPERADOR') || '',
      };
    }
    return {
      CDOPERADOR: localStorage.getItem('CDOPERADOR') || '',
      CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || 'T',
    };
  }
}
