import {
  Loader, Metadata, IDictionary, I18n, Http, Router, dayjs as moment,
} from '@zeedhi/core';
import {
  Form, DialogService, LoadingService, Select,
} from '@zeedhi/common';
import { TekLibAuth } from '@zeedhi/tek-lib';
import { getStoredData } from '@zeedhi/zeedhi-miragejs';
import { callMakeServer } from '../server/server';
import { getConfig } from '../server/config';
import { useMirage } from '../server/use-mirage';

export class ImportacaoController {
  private utilities = Loader.getInstance('Utilities');

  public unidades: IDictionary = this.utilities.getItemLocalStorage('Units') || [];

  public agrupamentosProdutos: IDictionary = this.utilities.getItemLocalStorage('GruposProduto') || [];

  public async importar() {
    const form = Metadata.getInstance<Form>('formImportacao');
    if (form.validate()) {
      if (form.value.UNIDADE.length <= 30) {
        if (form.value.INTERVAL.length === 2) {
          const { value } = form;
          if (this.utilities.validateInterval(value.INTERVAL[0], value.INTERVAL[1])) {
            if (localStorage.getItem('hasImportedData') !== 'true') {
              this.continueImportacao();
            } else this.hasImportedData();
          }
        } else this.utilities.showAlert('errorFilter', 'FILTER_2_DATES', 'error');
      } else this.utilities.showAlert('errorMaxFilial', 'ERROR_MAX_FILIAL', 'error');
    }
  }

  public continueImportacao() {
    this.validateActiveUser().then((valid) => {
      if (valid) {
        LoadingService.show(' ');
        const form = Metadata.getInstance<Form>('formImportacao');
        const { value } = form;
        const filialConcat = value.UNIDADE.map((u: any) => u.CDFILIAL).join('_');
        localStorage.setItem('FILIAIS_IMPORTACAO', filialConcat);
        const dataIni = moment(value.INTERVAL[0], 'YYYY-MM-DD').format('DD/MM/YYYY');
        localStorage.setItem('DATA_INICIAL', dataIni);
        const dataFin = moment(form.value.INTERVAL[1], 'YYYY-MM-DD').format('DD/MM/YYYY');
        localStorage.setItem('DATA_FINAL', dataFin);
        const cdgrupprod = value.AGRUPAMENTO_PRODUTO?.CDGRUPPROD ? value.AGRUPAMENTO_PRODUTO?.CDGRUPPROD : 'T';
        localStorage.setItem('CDGRUPPROD', cdgrupprod);
        const operador = localStorage.getItem('CDOPERADOR') || '';
        localStorage.setItem('CDOPERADOR_IMPORTACAO', operador);
        const serverConfig = getConfig(true);
        callMakeServer(serverConfig).then(() => {
          setTimeout(() => {
            const cdFilialLogada = localStorage.getItem('CDFILIALLOGADA');
            const select = Metadata.getInstance<Select>('CDFILIALLOGADA');
            select.datasource.get().then(() => {
              if (filialConcat.includes(cdFilialLogada)) {
                select.setValue(cdFilialLogada);
              } else {
                select.setValue(value.UNIDADE[0].CDFILIAL);
              }
            });
            getStoredData('produtos').then((result) => {
              let text = '';
              if (result) text = 'scm::SUCCESS_IMPORT';
              else text = 'scm::SUCCESS_IMPORT_NO_PRODUCTS';
              LoadingService.hide();
              DialogService.show({
                name: 'successImport',
                type: 'success',
                title: I18n.translate('scm::SUCCESS'),
                text: I18n.translate(text),
                buttons: [
                  {
                    name: 'ok',
                    component: 'ZdButton',
                    label: I18n.translate('scm::OK'),
                    events: {
                      click: () => {
                        localStorage.setItem('hasImportedData', 'true');
                        localStorage.setItem('request-log', '[]');
                        localStorage.setItem('request-error-log', '[]');
                        useMirage(true);
                        DialogService.hide();
                        Router.push('/');
                      },
                    },
                  },
                ],
              });
            });
          }, 20000);
        });
      }
    });
  }

  public validateActiveUser() {
    const filter = [
      {
        name: 'CDOPERADOR',
        operator: '=',
        value: localStorage.getItem('CDOPERADOR'),
      },
    ];
    LoadingService.show(' ');
    return Http.post('/OperadorAtivo', { requestType: 'FilterData', filter }).then((result: IDictionary) => {
      if (result.data?.dataset?.usuario?.ativo !== true) {
        // this.utilities.showAlert('inactiveUser', 'INACTIVE_USER', 'error');
        DialogService.show({
          name: 'errorInactiveUser',
          type: 'error',
          title: I18n.translate('scm::ERROR'),
          text: I18n.translate('scm::INACTIVE_USER'),
          buttons: [
            {
              name: 'ok',
              component: 'ZdButton',
              label: I18n.translate('scm::OK'),
              events: {
                click: () => {
                  DialogService.hide();
                  TekLibAuth.logout();
                },
              },
            },
          ],
        });
        return false;
      }
      return true;
    }).finally(() => {
      LoadingService.hide();
    });
  }

  public hasImportedData() {
    // this.utilities.showAlert('alreadyImported', 'HAS_IMPORTED_DATA', 'e')
    DialogService.show({
      name: 'confirmImport',
      type: 'warning',
      title: I18n.translate('scm::ATTENTION'),
      text: I18n.translate('scm::HAS_IMPORTED_DATA'),
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: I18n.translate('scm::YES'),
          events: {
            click: () => {
              DialogService.hide();
              this.continueImportacao();
            },
          },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: I18n.translate('scm::NO'),
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }

  public get getFilterFiliais() {
    return {
      CDOPERADOR: localStorage.getItem('CDOPERADOR') || '',
      CDFILIAL: 'T',
    };
  }

  public checkAccess() {
    if (!navigator.onLine) {
      DialogService.show({
        name: 'errorImport',
        type: 'error',
        title: I18n.translate('scm::ERROR'),
        text: I18n.translate('scm::IMPORT_CONNECTION_FAIL'),
        buttons: [
          {
            name: 'ok',
            component: 'ZdButton',
            label: I18n.translate('scm::OK'),
            events: {
              click: () => {
                DialogService.hide();
                Router.push('/');
              },
            },
          },
        ],
      });
    } else {
      useMirage(false);
    }
  }

  public beforeDestroy() {
    if (localStorage.getItem('hasImportedData') === 'true') {
      useMirage(true);
    } else useMirage(false);
  }
}
